/*
 * Created on Aug 19, 2004
 *
 */
package charlesgunn.anim.core;

import java.awt.Color;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collections;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import charlesgunn.anim.util.AnimationUtility;
import de.jreality.geometry.IndexedLineSetUtility;
import de.jreality.geometry.Primitives;
import de.jreality.math.CubicBSpline;
import de.jreality.math.FactoredMatrix;
import de.jreality.math.MatrixBuilder;
import de.jreality.math.P3;
import de.jreality.math.Pn;
import de.jreality.math.Quaternion;
import de.jreality.math.Rn;
import de.jreality.scene.Appearance;
import de.jreality.scene.IndexedLineSet;
import de.jreality.scene.SceneGraphComponent;
import de.jreality.scene.Transformation;
import de.jreality.shader.CommonAttributes;
import de.jreality.util.DefaultMatrixSupport;
import de.jreality.util.SceneGraphUtility;

/**
 * 
 *  @author gunn
 */
public class FramedCurve  {
	Vector<ControlPoint> controlPoints;
	double tmin, tmax;
	IndexedLineSet curveRepresentation;
	private boolean outOfDate;
	private SceneGraphComponent sgRep,
		camIcon,
		camIconItself,
		theCurveItself,
		currentPoint;
	private double iconScale = .02;
    private boolean useBSpline = true;
    int metric;
    double diameter;
    File sourceFile= null;
    int pointsForCurveRepn = 100;
    boolean useNatural = false;
    boolean interpolateScale = false;
    
	CubicBSpline cbx;
	CubicBSpline cby;
	CubicBSpline cbz;
	CubicBSpline cbr;
	CubicBSpline cbi;
	CubicBSpline cbj;
	CubicBSpline cbk;
	CubicBSpline cbsx, cbsy, cbsz;
	
	
	public FramedCurve()		{
		this(Pn.EUCLIDEAN);
	}
	
	public FramedCurve(int sig)		{
		super();
		metric = sig;
		controlPoints = new Vector<ControlPoint>();
		sgRep = SceneGraphUtility.createFullSceneGraphComponent("framedCurveRepn");
	}
		
	public static class ControlPoint	implements Comparable {
		private FactoredMatrix tt;
		double t;
			
		public int compareTo(Object o) {
			if ( t > ((ControlPoint) o).t) return 1;
			else if (t < ((ControlPoint) o).t) return -1;
			else return 0;
		}
		/**
		 * @param tt
		 * @param t
		 */
		public ControlPoint(FactoredMatrix tt, double t) {
			super();
			this.setTt(tt);
			this.t = t;
		}

		/**
		 * @param value
		 */
		public void setTime(double value) {
			t = value;
		}

		/**
		 * @return
		 */
		public double getTime() {
			return t;
		}
		
		public FactoredMatrix getTransformation() {
			return getTt();
		}
		public void setTt(FactoredMatrix tt) {
			this.tt = tt;
		}
		public FactoredMatrix getTt() {
			return tt;
		}
	}
	
	public static FramedCurve frameCurveFactory(ControlPoint[] cp)	{
		FramedCurve fc = new FramedCurve();
		SceneGraphComponent rep = fc.getSceneGraphRepresentation();
		Appearance ap = new Appearance();
		ap.setAttribute(CommonAttributes.LINE_SHADER+"."+CommonAttributes.DIFFUSE_COLOR, Color.white);
		ap.setAttribute(CommonAttributes.VERTEX_DRAW, false);
		ap.setAttribute(CommonAttributes.EDGE_DRAW, true);
		ap.setAttribute(CommonAttributes.TUBES_DRAW, false);
		ap.setAttribute(CommonAttributes.FACE_DRAW, false);
		ap.setAttribute(CommonAttributes.POINT_SHADER+"."+CommonAttributes.POINT_SIZE, 3.0);
		rep.setAppearance(ap);
		fc.setControlPoints(cp);
		return fc;
	}
	
     public void setControlPoints(FactoredMatrix[] cp)	{
		controlPoints.clear();
		int n = cp.length;
		for (int i  = 0; i<n; ++i)	{
			double p = (i==0) ? 0d : (i/(n-1.0));
	   		controlPoints.add( new ControlPoint(cp[i], p));	
		}
		outOfDate=true;
		update();
	}

    public void setControlPoints(double[][] matrices)	{
    		controlPoints.clear();
    		int n = matrices.length;
    		for (int i  = 0; i<n; ++i)	{
    			double p = (i/(n-1.0));
    	   		controlPoints.add(new ControlPoint(new FactoredMatrix((double[]) matrices[i].clone()), p));	
    		}
    		outOfDate=true;
    		update();
    }
    
    public void setControlPoints(ControlPoint[] cp)		{
    		controlPoints.clear();
    		int n = cp.length;
    		for (int i = 0; i<n; ++i)	controlPoints.add(cp[i]);
    		outOfDate = true;
    		update();
	}
    
    public void addControlPoint(ControlPoint cp)	{
    		controlPoints.add(cp);
    		outOfDate = true;
    		//update();
    }

	public double getTmax() {
		if (outOfDate) update();
		return tmax;
	}
	public double getTmin() {
		if (outOfDate) update();
		return tmin;
	}
	
    @SuppressWarnings("unchecked")
	private void update()		{
    		if (!outOfDate) return;
    		//JOGLConfiguration.theLog.log(Level.FINE
    		Collections.sort(controlPoints);
    		tmin = ((ControlPoint) controlPoints.firstElement()).t;
    		tmax = ((ControlPoint) controlPoints.lastElement()).t;
    		// extract translational part
       	double[] p0 = Rn.matrixTimesVector( null, ((ControlPoint) controlPoints.firstElement()).getTt().getArray(), P3.originP3);
       	double[] p1 = Rn.matrixTimesVector( null, ((ControlPoint) controlPoints.lastElement()).getTt().getArray(), P3.originP3);
       	diameter = Pn.distanceBetween(p0, p1, metric);
       	iconScale = .0002 * diameter;
    		int n = controlPoints.size();
    		Iterator iter;
    		int i;
    		if (useBSpline  && controlPoints.size() > 1)	{
    			double[] t = new double[n];
    			double[] x = new double[n];
    			double[] y = new double[n];
    			double[] z = new double[n];
    			double[] r = new double[n];
    			double[] qi = new double[n];
    			double[] qj = new double[n];
    			double[] qk = new double[n];
    			double[] sx = new double[n];
    			double[] sy = new double[n];
    			double[] sz = new double[n];
    			Quaternion oldQ = null;
   	   			//ControlPoint tmp = (ControlPoint) iter.next();
    			i = 0;
   	   		    for (ControlPoint  tmp : controlPoints)	{
//   	   		for ( i = 0, iter = controlPoints.iterator(); iter.hasNext(); ++i )	{
//    	   			ControlPoint tmp = (ControlPoint) iter.next();
        			 double[] m = tmp.getTt().getArray();
        			 Quaternion rot = tmp.getTt().getRotationQuaternion();
    				double w = m[15];
    				t[i] = tmp.t;
    				x[i] = m[3]/w;
    				y[i] = m[7]/w;
    				z[i] = m[11]/w;
    				if (oldQ != null) {
    					double dot = Quaternion.innerProduct(oldQ, rot);
    					dot = Math.max(-1.0, Math.min(1.0, dot));
    					System.err.println("angle is "+Math.acos(Math.abs(dot)));
    					if (dot < 0) Quaternion.times(rot, -1.0, rot);
    				}
    				r[i] = rot.re;
    				qi[i] = rot.x;
    				qj[i] = rot.y;
    				qk[i] = rot.z;
    				oldQ = rot;
    				if (interpolateScale)	{
    					double[] scale = tmp.getTt().getStretch();
    					sx[i] = scale[0];
    					sy[i] = scale[1];
    					sz[i] = scale[2];
    				}
    				i++;
    			}
    	   		if (useNatural)	{
        			cbx = new CubicBSpline.Natural(t, x);
        			cby = new CubicBSpline.Natural(t, y);
        			cbz = new CubicBSpline.Natural(t, z);
        			cbr = new CubicBSpline.Natural(t, r);
        			cbi = new CubicBSpline.Natural(t, qi);
        			cbj = new CubicBSpline.Natural(t, qj);
        			cbk = new CubicBSpline.Natural(t, qk);
        			if (interpolateScale)	{
            			cbsx = new CubicBSpline.Natural(t, sx);
            			cbsy = new CubicBSpline.Natural(t, sy);
            			cbsz = new CubicBSpline.Natural(t, sz);        				
        			}
    	   		} else {
    	   			cbx = new CubicBSpline.Default(t, x);
        			cby = new CubicBSpline.Default(t, y);
        			cbz = new CubicBSpline.Default(t, z);
        			cbr = new CubicBSpline.Default(t, r);
        			cbi = new CubicBSpline.Default(t, qi);
        			cbj = new CubicBSpline.Default(t, qj);
        			cbk = new CubicBSpline.Default(t, qk);  
        			if (interpolateScale)	{
            			cbsx = new CubicBSpline.Default(t, sx);
            			cbsy = new CubicBSpline.Default(t, sy);
            			cbsz = new CubicBSpline.Default(t, sz);        				
        			}
    	   		}

//   		for ( i = 0, iter = controlPoints.iterator(); iter.hasNext(); ++i )	{
//    			ControlPoint tmp = (ControlPoint) iter.next();
//    			Rn.matrixTimesVector(verts[i], tmp.tt.getMatrix(), P3.originP3);
//     	}
    		// construct the vertex list for the curve
    		pointsForCurveRepn = 16 * controlPoints.size();
    		double[][] verts = new double[pointsForCurveRepn][4];
    		double dt = (tmax - tmin)/pointsForCurveRepn;
    		for (i = 0; i<pointsForCurveRepn;++i)	{
    			double tt = tmin + dt * i;
    			verts[i][0] = cbx.valueAt(tt);
    			verts[i][1] = cby.valueAt(tt);
    			verts[i][2] = cbz.valueAt(tt);
    			verts[i][3] = 1.0;
    			//double w = cbw.valueAt(t);
    			
    		}
     	curveRepresentation = IndexedLineSetUtility.createCurveFromPoints(verts, false);
     	if (theCurveItself == null)
     	{
     		theCurveItself = SceneGraphUtility.createFullSceneGraphComponent();
     		sgRep.addChild(theCurveItself);
     	}
     	
		theCurveItself.setGeometry(curveRepresentation);
//		theCurveItself.getAppearance().setAttribute(CommonAttributes.VERTEX_DRAW, false);
//		theCurveItself.getAppearance().setAttribute(CommonAttributes.FACE_DRAW, true);
		
		camIcon = new SceneGraphComponent();
		sgRep.addChild(camIcon);
		camIconItself = cameraIcon(iconScale);
   		for ( i = 0, iter = controlPoints.iterator(); iter.hasNext(); ++i )	{
   			ControlPoint tmp = (ControlPoint) iter.next();
			SceneGraphComponent sgc = new SceneGraphComponent();
			sgc.setTransformation(new Transformation(tmp.getTt().getArray()));
			sgc.addChild(camIconItself);
			camIcon.addChild(sgc);
		}
   		DefaultMatrixSupport.getSharedInstance().storeDefaultMatrices(sgRep);
   		outOfDate = false;
    		}
    }
       
    public void printRotationValues()	{
    	double[] angles = new double[controlPoints.size()];
    	Quaternion oldQ = null;
    	Iterator itera = controlPoints.iterator();
   		for ( int i = 0; itera.hasNext(); ++i )	{
   			ControlPoint tmp = (ControlPoint) itera.next();
			 Quaternion rot = tmp.getTt().getRotationQuaternion();
			if (oldQ != null) {
				double dot = Quaternion.innerProduct(oldQ, rot);
				if (dot < 0) {
					Quaternion.times(rot, -1.0, rot);
					dot = -dot;
				}
				angles[i] = (i==0) ? (Math.acos(dot)) : (Math.acos(dot)+angles[i-1]);
//				System.err.println("angle is "+Math.acos(dot));
			}
			oldQ = rot;
		}
   		double total = angles[angles.length-1];
   		for (int i = 0; i<angles.length; ++i)	{
   			angles[i] /= total;
   			System.err.println("parameter is: "+angles[i]);
   		}

    }
    public void setUnitSpeed()	{
    		double length = 0.0;
         Iterator iter;
    		int i;
    		double[] pt = new double[4];
    		double[] opt = new double[4];
    		for ( i = 0, iter = controlPoints.iterator(); iter.hasNext(); ++i )	{
    			ControlPoint cp = (ControlPoint) iter.next();
   			double m[] = cp.getTt().getArray();
			pt[0] = m[3]; pt[1] = m[7]; pt[2] = m[11];  pt[3] = m[15];
   			if (i>0)	{
     			double d =  Pn.distanceBetween(pt, opt, metric);
    				cp.t  = length = length + d;
    			} else cp.t = 0.0;
    			System.arraycopy(pt, 0, opt, 0, 4);
    		}
		tmin = 0.0;
		tmax = length;
		outOfDate=true;
    }
    
    private int previousSegment = -20;
    public int getSegmentAtTime(double t)	{
   		if (outOfDate) update();
		// check to see if its in the previous segment: usually will be
   	   	if (previousSegment >= 0 && previousSegment <= (controlPoints.size() -1))	{
   			if (t >= ((ControlPoint) controlPoints.get(previousSegment)).getTime() && 
   					t < ((ControlPoint) controlPoints.get(previousSegment+1)).getTime()) 
   				return previousSegment;
   		}
   		Iterator iter;
	    	int i;
	    	if (t < tmin) return -1;
	    	if (t > tmax) return controlPoints.size();
	    	for ( i = 0, iter = controlPoints.iterator(); iter.hasNext(); ++i )	{
	    		ControlPoint cp = (ControlPoint) iter.next();
	    		if (t <= cp.t)   break;
	    	}
	    	if (i==0) return -1;
	    previousSegment = i-1;
	    return previousSegment;
    }
    
	public FactoredMatrix getValueAtTime(double time) {
		return getValueAtTime(time, null);
	}
	
   public FactoredMatrix getValueAtTime(double t, FactoredMatrix dst)		{
	   //if (dst == null) 
		   dst = new FactoredMatrix();
    		if (controlPoints.size() == 1) 	{
    			ControlPoint cp = ((ControlPoint) controlPoints.get(0));
//    			try {
//					dst = ((FactoredMatrix) cp.tt.clone());
//				} catch (CloneNotSupportedException e) {
//					// TODO Auto-generated catch block
//					e.printStackTrace();
//				}
    			dst.assignFrom(cp.getTt().getArray());
    			return dst;
    		}
    		// assume 0<=t<=1  
    		// assume each segment is same length in parameter
    		if (outOfDate) update();
    		int whichSeg = getSegmentAtTime(t);
    		if (whichSeg == -1) {whichSeg = 0; t = tmin;}
    		else if (whichSeg >= controlPoints.size()-1) {t = tmax; whichSeg = controlPoints.size()-2; }
        	ControlPoint cp1 = (ControlPoint) controlPoints.get(whichSeg);
       	ControlPoint cp2 = (ControlPoint) controlPoints.get(whichSeg+1);
       	double dt = cp2.t - cp1.t;
       	if (currentPoint == null) {
       		currentPoint = new SceneGraphComponent();
       		currentPoint.setTransformation(new Transformation());
       		currentPoint.addChild(camIconItself);
       		camIcon.addChild(currentPoint);
       	}	// else ??
		double[] trans = new double[3], scale = new double[3];
		Quaternion rot = new Quaternion();
		if (useBSpline)	{
			trans[0] = cbx.valueAt(t);
			trans[1] = cby.valueAt(t);
			trans[2] = cbz.valueAt(t);
	      	dst.setTranslation(trans);		
			rot.re = cbr.valueAt(t);
			rot.x = cbi.valueAt(t);
			rot.y = cbj.valueAt(t);
			rot.z = cbk.valueAt(t);
			Quaternion.normalize(rot, rot);
	      	dst.setRotation(rot);	
	      	if (interpolateScale)	{
	      		scale[0] = cbsx.valueAt(t);
	      		scale[1] = cbsy.valueAt(t);
	      		scale[2] = cbsz.valueAt(t);
	      	}
		} else {
	     	AnimationUtility.linearInterpolation(dst, cp1.getTt(), cp2.getTt(), (t - cp1.t)/dt);
		}
       	dst.assignTo(currentPoint.getTransformation());	 			
     	return dst;
    }
    
   public static FramedCurve readFromFile(String filename)		{
	File file = new File(filename);
	return readFromFile(file);
}

public static FramedCurve readFromFile(File file)		{
	FramedCurve fc = null;
	try {
		//File file = new File(filename);
		BufferedReader fin = new BufferedReader(new FileReader(file));
		fc = readFromFile(fin);
		fin.close();
		}
		catch (java.io.IOException ev)	{
			ev.printStackTrace();
		}
		return fc;
    }
    
   public static FramedCurve readFromFile(BufferedReader fin)		{
			String ss;
			double[] mat = new double[16];
			double thist = 0;
			int count = 0;
			Vector<ControlPoint> cplist = new Vector<ControlPoint>();
			int metric = Pn.EUCLIDEAN;
			boolean firsttime = true;
			try {
				while ( (ss = fin.readLine()) != null)	{
					StringTokenizer st = new java.util.StringTokenizer(ss);
					while (st.hasMoreTokens())	{
						String firstT = st.nextToken();
						if (firstT.charAt(0) == '#') continue;
						if (count == 0) thist = Double.parseDouble(firstT);
						else mat[count-1] = Double.parseDouble(firstT);
						count++;
						if ((count % 17) == 0)	{	// read one matrix
							if (firsttime && metric == Pn.PROJECTIVE)	{	// first matrix!
								if (mat[12] == 0.0 && mat[13] == 0.0 && mat[14] == 0.0 && mat[15] == 1.0) metric = Pn.EUCLIDEAN;
								else {		// try to figure out the curvature;
									double[] row1 = new double[4];
									System.arraycopy(mat, 0, row1, 0, 4);
									if (Rn.innerProduct(row1, row1) > 1.01) metric = Pn.HYPERBOLIC;
									else metric = Pn.ELLIPTIC;
								}
								firsttime = false;
							}
							count = 0;
							FactoredMatrix gen = new FactoredMatrix(metric, (double[]) mat.clone());
							ControlPoint thiscp = new ControlPoint(gen, thist);
							cplist.add(thiscp);
						}
					}
				}
			} catch (NumberFormatException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			ControlPoint[] garray = new ControlPoint[0];
			garray = (ControlPoint[]) cplist.toArray(garray);
			FramedCurve fc = frameCurveFactory(garray);
			return fc;
	
}
    public  void writeToFile(String filename)		{
		if (outOfDate) update();
		File file = new File(filename);
		writeToFile(file);
    }
    
    public  void writeToFile(File file)		{
    		if (outOfDate) update();
 		try {
	        PrintWriter pw = new PrintWriter(new FileWriter(file));
	        Iterator iter;
	   		int i;
	   		for ( i = 0, iter = controlPoints.iterator(); iter.hasNext(); ++i )	{
	   			ControlPoint tmp = (ControlPoint) iter.next();
	   			pw.println(tmp.t);
	   			pw.print(Rn.matrixToString(tmp.getTt().getArray()));
			}
			pw.close();
 		}
		catch (java.io.IOException ev)	{
			ev.printStackTrace();
		}
    }
    
    public ControlPoint getControlPoint(int i)	{
    		ControlPoint cp = null;
    		i = i%controlPoints.size();
    		if (outOfDate) update();
    		cp = (ControlPoint) controlPoints.elementAt(i);
    		return cp;
    }

	/**
	 * @param inspectedPoint
	 */
	public void deleteControlPoint(int i) {
		if (outOfDate) update();
		if (i>=controlPoints.size()) return;
		if (i<0) return;
		controlPoints.remove(i);
		outOfDate = true;
	}
	/**
	 * @param b
	 */
	public void setOutOfDate(boolean b) {
		outOfDate = b;
		
	}

	/**
	 * @return
	 */
	public int getNumberControlPoints() {
		return controlPoints.size();
	}
	public File getSourceFile() {
		return sourceFile;
	}

	public void setSourceFile(File sourceFile) {
		this.sourceFile = sourceFile;
	}
	
	public void multiplyOnLeft(double[] mat)	{
		Iterator iter;
	    	for (iter = controlPoints.iterator(); iter.hasNext(); )	{
	    		ControlPoint cp = (ControlPoint) iter.next();
	    		cp.getTt().multiplyOnLeft(mat);
	    	}
	}

	/**
	 * @param mat
	 */
	public void repair(double[] mat1, double[] mat2) {
		Iterator iter;
	    	for (iter = controlPoints.iterator(); iter.hasNext(); )	{
	    		ControlPoint cp = (ControlPoint) iter.next();
	    		//cp.tt.setMatrix(Rn.conjugateByMatrix(null, cp.tt.getMatrix(), mat));
	    		double [] tr = cp.getTt().getArray();
	    		boolean toOriginal = false;
	    		if (toOriginal)	{
		    		tr[3] = tr[3] + mat1[3]; // - mat2[3];
		    		tr[7] = tr[7] + mat1[7]; // - mat2[7];
		    		tr[11] = tr[11] + mat1[11]; // - mat2[11];	
	    		} else {
		    		tr[3] = tr[3] - mat2[3];
		    		tr[7] = tr[7] - mat2[7];
		    		tr[11] = tr[11]  - mat2[11];	
	    			
	    		}
	    		cp.getTt().assignFrom(tr);
	    	}
	}

	public boolean isInterpolateScale() {
		return interpolateScale;
	}

	public void setInterpolateScale(boolean interpolateScale) {
		this.interpolateScale = interpolateScale;
	}

	public SceneGraphComponent getSceneGraphRepresentation() {
		return sgRep;
	}

	public void setSceneGraphRepresentation(SceneGraphComponent sgRep) {
		this.sgRep = sgRep;
	}

	public void scaleBy(double scale) {
		//HACK
		for (ControlPoint cp : controlPoints)	{
			FactoredMatrix fm = cp.getTransformation();
			double[] tlate = fm.getTranslation();
			for (int i = 0; i<3; ++i) tlate[i] *= scale;
			fm.setTranslation(tlate);
		}
		
	}

	private static SceneGraphComponent _camIcon = null;
	static {
		_camIcon = SceneGraphUtility.createFullSceneGraphComponent("cameraIcon");
		_camIcon.getAppearance().setAttribute(CommonAttributes.VERTEX_DRAW, false);
		_camIcon.getAppearance().setAttribute(CommonAttributes.EDGE_DRAW, true);
//		_camIcon.getAppearance().setAttribute(CommonAttributes.FACE_DRAW, false);
		_camIcon.getAppearance().setAttribute(CommonAttributes.LINE_SHADER+"."+CommonAttributes.DIFFUSE_COLOR, Color.white);
		_camIcon.setGeometry(Primitives.cube());
//		MatrixBuilder.euclidean().scale(.3,1.0,1.0).assignTo(_camIcon.getTransformation());

		SceneGraphComponent viewer = new SceneGraphComponent();
		_camIcon.addChild(viewer);
	}
	public static SceneGraphComponent cameraIcon(double scale)		{
		SceneGraphComponent ci = SceneGraphUtility.createFullSceneGraphComponent("cameraIcon");
		MatrixBuilder.euclidean().scale(scale).assignTo(ci);
		ci.addChild(_camIcon);
		return ci;
	}

	public boolean isUseBSpline() {
		return useBSpline;
	}

	public void setUseBSpline(boolean useBSpline) {
		this.useBSpline = useBSpline;
	}
	

 }
