package charlesgunn.anim.core;


/**
 * Key framing requires the ability to add and delete key frames
 * (editing is done by adding keyframe at an existing TimeDescriptor).
 * Also, need to be able to set the current value in advance of
 * setting a key frame. (For manual setting of key frames, for example).
 * @author Charles Gunn
 *
 * @param <T>  T is the type of the instance which is being animated, e.g., Double.
 */
public interface KeyFrameAnimated<T> extends Animated {
	public void deleteKeyFrame(TimeDescriptor time);
	// following allows key frames to be set when the value has been set via another route
	public void addKeyFrame(TimeDescriptor td);
	// want to be able to directly set the value before e.g. setting key frame
	// warning: this only works for immutable classes. 
	// if mutable, then write the new value into the target instance before adding key frame
	public void setCurrentValue(T value);
	// there may be key frame animated values that are only active on playback, but not for
	// setting key frames
	public boolean isWritable();
	public void setWritable(boolean b);
	// a flag which says whether setting a key frame at time t with old  value v automatically
	// overwrites all key frames "to the right" with the same value v.
	public boolean isGivesWay();
	public void setGivesWay(boolean b);
	// Here's a method to support saving/restoring
	public void setFromArchive(KeyFrameAnimated<T> archived);
}
