package charlesgunn.anim.core;

import java.awt.Color;

import charlesgunn.anim.util.AnimationUtility;

/**
 * Animate an instance of {@link Color}.
 * @author Charles Gunn
 *
 */
public class KeyFrameAnimatedColor extends SimpleKeyFrameAnimated<Color> {

	public KeyFrameAnimatedColor(Color c) {
		super(c);
	}

	public KeyFrameAnimatedColor(KeyFrameAnimatedDelegate<Color> ad) {
		super(ad);
	}

	@Override
	public Color getNewInstance() {
		return new Color(255, 255, 255);
	}

	@Override
	public Color copyFromTo(Color from, Color to) {
		return from;
	}

	@Override
	public void setValueAtTime(double t) {
		if (getKeyFrames().size() == 0) return;
		super.setValueAtTime(t);
		if (!valueIsSet)	{// superclass handles out-of-range times
			if (interp == AnimationUtility.InterpolationTypes.LINEAR)
				currentValue = AnimationUtility.linearInterpolation(
						t, previous.getTime(),next.getTime(),previous.getValue(), next.getValue());
			else //if (interp == AnimationUtility.InterpolationTypes.CUBIC_HERMITE)
				currentValue = AnimationUtility.hermiteInterpolation(
						t, previous.getTime(),next.getTime(),previous.getValue(), next.getValue());
		}
		delegate.propagateCurrentValue(currentValue);
	}

	@Override
	public boolean equalValues(Color t1, Color t2) {
		// TODO Auto-generated method stub
		return t1.getRed() == t2.getRed() && t1.getGreen() == t2.getGreen() && t1.getBlue() == t2.getBlue() && t1.getAlpha() == t2.getAlpha();
	}



}
