package charlesgunn.anim.core;

import charlesgunn.anim.util.AnimationUtility;

/**
 * Fill in the required methods to animate a variable of type {@link Double}.
 * @author Charles Gunn
 *
 */
public class KeyFrameAnimatedDouble extends SimpleKeyFrameAnimated<Double> {

	public KeyFrameAnimatedDouble(Double d) {
		super(d);
	}

	public KeyFrameAnimatedDouble(KeyFrameAnimatedDelegate<Double> ad) {
		super(ad);
	}

	@Override
	public Double getNewInstance() {
		return new Double(0.0);
	}

	@Override
	public void setValueAtTime(double t) {
		if (getKeyFrames().size() == 0) return;
		super.setValueAtTime(t);
		if (!valueIsSet)	{// superclass handles out-of-range times
			if (interp == AnimationUtility.InterpolationTypes.LINEAR)
				currentValue = AnimationUtility.linearInterpolation(
						t, previous.getTime(),next.getTime(),previous.getValue(), next.getValue());
			else if (interp == AnimationUtility.InterpolationTypes.CUBIC_HERMITE)
				currentValue = AnimationUtility.hermiteInterpolation(
						t, previous.getTime(),next.getTime(),previous.getValue(), next.getValue());
		}
		delegate.propagateCurrentValue(currentValue);
	}

	@Override
	public Double copyFromTo(Double from, Double to) {
		return from;
	}

	@Override
	public boolean equalValues(Double t1, Double t2) {
		return t1.doubleValue() == t2.doubleValue();
	}



}
