package charlesgunn.anim.core;

import charlesgunn.anim.util.AnimationUtility;

/**
 * Fill in the required methods to animate a variable of type {@link Double}.
 * @author Charles Gunn
 *
 */
public class KeyFrameAnimatedInteger extends SimpleKeyFrameAnimated<Integer> {

	public KeyFrameAnimatedInteger(Integer d) {
		super(d);
	}

	public KeyFrameAnimatedInteger(KeyFrameAnimatedDelegate<Integer> ad) {
		super(ad);
	}

	@Override
	public Integer getNewInstance() {
		return new Integer(0);
	}

	@Override
	public void setValueAtTime(double t) {
		if (getKeyFrames().size() == 0) return;
		super.setValueAtTime(t);
		if (!valueIsSet)	{ // superclass handles out-of-range times
			switch(interp)	{
				case CONSTANT:
					currentValue = previous.getValue();
					break;
				case LINEAR:
					currentValue = (int) AnimationUtility.linearInterpolation(
							t, previous.getTime(),next.getTime(),(double)previous.getValue(),(double) next.getValue());
					break;
				case CUBIC_HERMITE:
				default:
					currentValue = (int) AnimationUtility.hermiteInterpolation(
							t, previous.getTime(),next.getTime(), (double) previous.getValue(), (double) next.getValue());
						break;
			}
		}
		delegate.propagateCurrentValue(currentValue);
	}

	@Override
	public Integer copyFromTo(Integer from, Integer to) {
		return from;
	}

	@Override
	public boolean equalValues(Integer t1, Integer t2) {
		return t1.intValue() == t2.intValue();
	}


}
