package charlesgunn.anim.core;

import de.jreality.math.FactoredMatrix;
import de.jreality.math.Pn;
import de.jreality.math.Rn;
import de.jreality.scene.Transformation;

/**
 * Animate an instance of {@link Transformation} by providing a delegate to
 * an instance of {@link KeyFrameAnimatedIsometry} that copies the 4x4 matrix
 * from one place to another, i.e, from <i>target</i> to and from <i>tform</i>.
 * 
 * @author Charles Gunn
 *
 */
public class KeyFrameAnimatedTransformation extends KeyFrameAnimatedIsometry {
	transient Transformation tform;

	public KeyFrameAnimatedTransformation(Transformation t)	{
		this(t, Pn.EUCLIDEAN);
	}
	
	public KeyFrameAnimatedTransformation(Transformation t, int metric)	{
		super(new FactoredMatrix(metric, t.getMatrix()));
		tform = t;
		currentValue.assignFrom(t.getMatrix());
	}
	
	public KeyFrameAnimatedTransformation(
			KeyFrameAnimatedDelegate<FactoredMatrix> animatedDelegate) {
		super(animatedDelegate);
	}

	public KeyFrameAnimatedDelegate< FactoredMatrix>  getDefaultDelegate() {
		if (target == null) return null;
		return new KeyFrameAnimatedDelegate<FactoredMatrix>() {

			public void propagateCurrentValue(FactoredMatrix fm) {
				if (!tform.isReadOnly()) tform.setMatrix(fm.getArray());
			}

			public FactoredMatrix gatherCurrentValue(FactoredMatrix fm) {
				fm.assignFrom(tform.getMatrix());
				return fm;
			}
			
		};
	}

	@Override
	public void printState() {
		int i = 0;
		for (KeyFrame<FactoredMatrix> kf : getKeyFrames())	{
			double[] m = kf.getValue().getArray();
			System.err.println("Time = "+kf.getTime()+"\tKey frame "+i+" = "+Rn.matrixToString(m));
			i++;
		}
	}

}
