/*
 * Created on Jan 4, 2008
 *
 */
package charlesgunn.anim.core;

import java.util.LinkedList;

/**
 * This is basically a sorted list of instances of {@link KeyFrame}, sorted by the 
 * value of {@link TimeDescriptor#getTime()}. 
 * 
 * TODO: check to make sure something reasonable is done if a TimeDescriptor is attempted 
 * to be added that has the same time value as an existing one.
 * 
 * @author Charles Gunn
 *
 * @param <T>
 */
public class SortedKeyFrameList<T> extends LinkedList<KeyFrame<T>> {
	
	public double getTmin()	{
		if (size() == 0) return 0.0;
		return get(0).getTime();
	}
	
	public double getTmax()	{
		if (size() == 0) return 0.0;
		return get(size()-1).getTime();
	}
	
	public KeyFrame<T> existingKeyFrameAt(TimeDescriptor td)	{
		for (KeyFrame<T> kf : this)	{
			if (kf.getTimeDescriptor() == td) return kf;
		}
		return null;
	}
	
	public void sortedInsert(KeyFrame<T> kf)	{
		int n = size();
		int i = 0;
		for (i = 0; i<n; ++i)	{
			if (get(i).getTime() > kf.getTime()) break;
		}
		if (i == 0) {
			add(0, kf);
			return;
		}
		add(i, kf);
	}
	@Override
	public boolean add(KeyFrame<T> k)	{
		int n = size();
		int i = 0;
		for (i = 0; i<n; ++i)	{
			if (get(i).getTime() > k.getTime()) break;
		}
		super.add(i, k);
		return true;
	}

	public int getSegmentAtTime(double t) {
		int n = size();
		int i = 0;
 		for (i = 0; i<n; ++i)	{
			if (t < get(i).getTime()) return i-1;
		}
		return n-1;
	}
}
