package charlesgunn.anim.gui;

import java.util.List;
import java.util.Vector;

import charlesgunn.anim.core.Animated;
import charlesgunn.anim.core.KeyFrameAnimated;
import de.jreality.scene.Scene;
import de.jreality.scene.Viewer;

public class AnimationPanelListenerImpl implements AnimationPanelListener {

	String name = "unnamed";
	List<Animated> animated = new Vector<Animated>();
	private List<KeyFrameAnimated> animatedKF = new Vector<KeyFrameAnimated>();
	
	public AnimationPanelListenerImpl(Viewer v, String string) {
		name = string;
		viewer = v;
	}

	public void printState() {
		for (Animated at : animated)	{
				at.printState();
		}				
	}

	public void actionPerformed(final AnimationPanelEvent e) {
			switch(e.type)	{
			case PLAYBACK_STARTED:
				for (Animated at : animated) at.startAnimation();
				break;
			case PLAYBACK_COMPLETED:
				System.err.println("sending end animation");
				for (Animated at : animated) at.endAnimation();
				break;
			case KEY_FRAME_ADDED:
			case KEY_FRAME_CHANGED:
				getAnimatedKF();
				for (KeyFrameAnimated at : animatedKF) at.addKeyFrame(e.time);
				break;
			case KEY_FRAME_DELETED:
				getAnimatedKF();
				for (KeyFrameAnimated at : animatedKF) at.deleteKeyFrame(e.time);
				break;
			case SET_VALUE_AT_TIME:
				Runnable runnable = new Runnable() {
					public void run() {
						for (Animated at : animated) {
							at.setValueAtTime(e.time.getTime());	
//							System.err.println(at.getName()+" Setting value at time "+e.time.getTime());
						}
					}
				};
				Scene.executeWriter(viewer.getSceneRoot(), runnable);
				break;
		}
	}
	private void getAnimatedKF() {
		animatedKF.clear();
		for (Animated aa : animated)	{
			if (aa instanceof KeyFrameAnimated) {
				animatedKF.add((KeyFrameAnimated) aa);
			}
		}
	}
	transient Viewer viewer;

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public List<Animated> getAnimated() {
		return animated;
	}

	public void setAnimated(List<Animated> animated) {
		this.animated = animated;
	}

	public Object getState() {
		return animated;
	}

	public void setState(Object o) {
		if ( !(o instanceof List))
			throw new IllegalArgumentException("wrong class");
		for (Object oo : (List) o)	{
			if ( !(oo instanceof KeyFrameAnimated))
				continue;
			KeyFrameAnimated kfa = (KeyFrameAnimated) oo;
			Animated original = elementNamed(kfa.getName(), animated);
			if (original == null || !(original instanceof KeyFrameAnimated)) {
				System.err.println("No original named "+kfa.getName());
				continue;
			}
			KeyFrameAnimated newkfa = (KeyFrameAnimated) original;
			System.err.println("Setting from archive "+kfa.getName());
			newkfa.setFromArchive(kfa);
		}
	}
	
	public static Animated elementNamed(String name, List<Animated> list)	{
		for (Animated kfa : list)	{
			String name2 = kfa.getName();
			if (name2 == null) {
				System.err.println("animated has no name: "+kfa);continue;
			}
			if (name2.equals(name)) return kfa;
		}
		return null;
	}

}
