package charlesgunn.anim.gui;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.List;
import java.util.Vector;

import javax.imageio.ImageIO;

import charlesgunn.anim.core.KeyFrameAnimated;
import de.jreality.scene.Scene;
import de.jreality.scene.Viewer;
import de.jreality.util.CameraUtility;
import de.jreality.util.ImageUtility;

public class AnimationPanelRecordListener implements AnimationPanelListener {

	String name = "recorder";
	transient Viewer viewer;
	transient private Dimension currentDim;
	transient BufferedImage img  = null, img2 = null;
	
	public AnimationPanelRecordListener(Viewer v, String string) {
		name = string;
		viewer = v;
	}


	public void actionPerformed(final AnimationPanelEvent e) {
		
			switch(e.type)	{
				case PLAYBACK_STARTED:
					if (e.source.isRecording()) {
						boolean isStereo = CameraUtility.getCamera(viewer).isStereo();
						currentDim = e.source.getRecordPrefs().getDimension();
						if (isStereo) {
							currentDim = new Dimension(currentDim.width * 2,
									currentDim.height);
						}
					}
					break;
				case SET_VALUE_AT_TIME:
					if (e.source.isRecording())	{
						RecordingPreferences rc = e.source.getRecordPrefs();
						double time = e.source.getCurrentTime(),
							start = rc.getStartTime(),
							end = rc.getEndTime();
						if ( (start >= 0 && start > time) ||
								(end >= 0 && end < time)) return;
						
						int tick = e.source.getCurrentFrame();
						System.err.println("current tick = "+tick+" time = "+ time+" fps = "+e.source.fps);
						String num = String.format("%04d",tick+rc.getStartCount());
						File outfile = new File(e.source.getRecordPrefs().getCurrentDirectoryPath()+"/"+e.source.getRecordPrefs().getStemName()+num+"."+
								e.source.getRecordPrefs().getFileFormatSuffix());
						System.err.println("outfile = "+outfile.getAbsolutePath());
						int antialiasing = e.source.getRecordPrefs().getAntialiasing();
						if (e.source.getRecordPrefs().isSaveScreenshot())	{
							img = ImageUtility.captureScreenshot(viewer);
						} else {
							if ( !(viewer instanceof de.jreality.jogl.JOGLViewer)) 
								throw new IllegalArgumentException("Can only render offscreen with jogl backend");
							img = ((de.jreality.jogl.AbstractViewer) viewer).renderOffscreen(
									img, antialiasing*currentDim.width, antialiasing*currentDim.height, antialiasing);
//							if (img2 == null || img2.getWidth() != currentDim.getWidth() || img2.getHeight() != currentDim.getHeight()) {
//								 img2 = new BufferedImage(
//									currentDim.width, 
//									currentDim.height,
//									e.source.getRecordPrefs().isSaveAlpha() ? BufferedImage.TYPE_INT_ARGB :  BufferedImage.TYPE_INT_RGB);
//							}
//							Graphics2D g = (Graphics2D) img2.getGraphics();
//							g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f));							g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
//							g.setColor(new Color(0, 0, 0, 0));
//							g.clearRect(0, 0, currentDim.width, currentDim.height);
//							g.drawImage(img, 0,0,currentDim.width,currentDim.height, null);							
//							img2.getGraphics().drawImage(
//									img.getScaledInstance(
//											currentDim.width,
//											currentDim.height,
//											BufferedImage.SCALE_SMOOTH
//									),
//									0,
//									0,
//									null
//							);
//							img = img2;
//							img.coerceData(true);
//							ImageIO.setUseCache(false);
						}
						ImageUtility.writeBufferedImage(outfile, img);
						System.err.println("Rendered "+outfile.getName());
					}
				}
	}


	public Object getState() {
		return null;
	}

	public void setState(Object o) {
	}

	public String getName() {
		return name;
	}

	public void printState() {
		
	}
	
}
