/**
 *
 * This file is part of jReality. jReality is open source software, made
 * available under a BSD license:
 *
 * Copyright (c) 2003-2006, jReality Group: Charles Gunn, Tim Hoffmann, Markus
 * Schmies, Steffen Weissmann.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * - Neither the name of jReality nor the names of its contributors nor the
 *   names of their associated organizations may be used to endorse or promote
 *   products derived from this software without specific prior written
 *   permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

package charlesgunn.anim.gui;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.jreality.util.Secure;
import de.jtem.beans.DimensionPanel;


/**
  */
public class RecordingPreferences {

	final static String[] fileSuffixes = {"tiff","jpg","png"};
	
	String stemName = "test",
		currentDirectoryPath = Secure.getProperty("charlesgunn.anim.writedir", Secure.getProperty("user.home")),
		fileFormatSuffix = fileSuffixes[2],
		frameCountFormatString = "%04d";
	private int antialiasing = 4,
		fps = 30,
		startCount = 0;
	private boolean saveAlpha = true, saveScreenshot = false;
	private double startTime = -1, endTime = -1;
	private Dimension dimension = new Dimension(1920, 1080);
	
	transient private DimensionPanel dimPanel;
	transient private JComponent panel;
	transient JFileChooser dirchooser;
	transient JTextField saveDirectoryTF, stemTF;
	transient private JRadioButton anti1B, anti2B, anti4B;
	transient private ButtonGroup antiBG;
	transient private JRadioButton suffixB[];
	transient private ButtonGroup suffixBG;
	transient private JTextField fpsTF, startTTF, endTTF;
	transient private JCheckBox saveAlphaCB, saveScreenshotCB;
	
	public RecordingPreferences()	{
		String propname = "charlesgunn.anim.writedir";
		String writeDir = Secure.getProperty(propname, Secure.getProperty("user.home"));
		System.err.println("anim writedir = "+writeDir);
		setCurrentDirectoryPath(writeDir);
		init();
	}
	
	private void init()	{
		suffixB = new JRadioButton[3];
		panel = new JPanel();
		panel.add(createAccessory());
	}
	public void setDimension(Dimension dim)	{
		dimension = dim;
		dimPanel.setDimension(dimension);
	}
	
	public void setAntialiasing(int a)	{
		antialiasing = a;
		if (antialiasing == 1) anti1B.setSelected(true);
		else if (antialiasing == 2) anti2B.setSelected(true);
		else anti4B.setSelected(true);
	}
	
	public void setFrameRate(int fps)	{
		this.fps = fps;
	}
	
	public String getStemName() {
		return stemName;
	}
	public void setStemName(String stemName) {
		this.stemName = stemName;
		stemTF.setText(stemName);
	}
	public String getCurrentDirectoryPath() {
		return currentDirectoryPath; //dirchooser.getCurrentDirectory().getAbsolutePath();
	}
	public void setCurrentDirectoryPath(String wd) {
		currentDirectoryPath = wd;
		if (dirchooser == null)
			dirchooser = new JFileChooser(currentDirectoryPath);
		dirchooser.setCurrentDirectory(new File(currentDirectoryPath));
		System.err.println("setting dirchooser to "+currentDirectoryPath);
	}
	
	public int getFps() {
		return fps;
	}
	public void setFps(int fps) {
		this.fps = fps;
		fpsTF.setText(Integer.toString(fps));
	}

	public boolean isSaveAlpha() {
		return saveAlpha;
	}

	public void setSaveAlpha(boolean saveAlpha) {
		this.saveAlpha = saveAlpha;
	}
	
	public int getAntialiasing() {
		return antialiasing;
	}
	public Dimension getDimension() {
		dimension = dimPanel.getDimension();
		System.err.println("got dimension "+dimension.toString());
		return dimension;
	}
	public double getStartTime() {
		return startTime;
	}

	public void setStartTime(double startTime) {
		this.startTime = startTime;
	}

	public double getEndTime() {
		return endTime;
	}

	public void setEndTime(double d) {
		this.endTime = d;
	}

	public JComponent getComponent()	{
		return panel;
	}
	
	private JComponent createAccessory() {
		
		if (dimPanel == null) {
			dimPanel = new DimensionPanel();
			dimPanel.setDimension(dimension);
			dimPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Dimension"));
			dimPanel.setToolTipText("Set image dimensions");
			dimPanel.setPreferredSize(new Dimension(200,100));
			dimPanel.addChangeListener(new ChangeListener() {
				
				@Override
				public void stateChanged(ChangeEvent arg0) {
					setDimension(dimPanel.getDimension());
				}
			});
		}
		
		Box accessory = Box.createVerticalBox();
		Box hbox = Box.createHorizontalBox();
		accessory.add(hbox);
		accessory.add(Box.createVerticalGlue());
		hbox.add(dimPanel);
		hbox.add(Box.createHorizontalGlue());
		Box vbox = Box.createVerticalBox();
		JPanel p = new JPanel();
		antiBG = new ButtonGroup();
		anti1B = new JRadioButton(new AbstractAction("none") {
			public void actionPerformed(ActionEvent e) {
				antialiasing = 1;
			}
		});
		antiBG.add(anti1B);
		p.add(anti1B);
		anti2B = new JRadioButton(new AbstractAction("2x2") {
			public void actionPerformed(ActionEvent e) {
				antialiasing = 2;
			}
		});
		antiBG.add(anti2B);
		p.add(anti2B);
		anti4B = new JRadioButton(new AbstractAction("4x4") {
			public void actionPerformed(ActionEvent e) {
				antialiasing = 4;
			}
		});
		antiBG.add(anti4B);
		setAntialiasing(antialiasing);
		p.add(anti4B);
		
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Antialiasing factor"));
		p.setToolTipText("<html><body>Choose the factor of dimension scaling<br>" +
				"for \"antialiased\" offscreen rendering</body></html>");
		vbox.add(p);
		
		p = new JPanel();
		suffixBG = new ButtonGroup();
		for (int i = 0; i<3; ++i)	{
			final int which = i;
			suffixB[i] = new JRadioButton(new AbstractAction(fileSuffixes[i]) {
				public void actionPerformed(ActionEvent e) {
					fileFormatSuffix = fileSuffixes[which];
				}
			});
			suffixBG.add(suffixB[i]);
			p.add(suffixB[i]);
		}
		suffixB[2].setSelected(true);
		fileFormatSuffix = fileSuffixes[2];
		
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Image format"));
		p.setToolTipText("<html><body>Choose the output image format<br>");
		vbox.add(p);
		
		hbox.add(vbox);
		
		vbox = Box.createVerticalBox();
		p = new JPanel();
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "FPS"));
		fpsTF = new JTextField(Integer.toString(fps), 5);
		fpsTF.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				fps = Integer.parseInt(fpsTF.getText());
			}
			
		});
		p.add(fpsTF);
		p.setToolTipText("<html><body>Set frames per second <br>" +
		"for recording mode</body></html>");
		hbox.add(Box.createHorizontalGlue());
		hbox.add(vbox);
		vbox.add(p);
		p = new JPanel();
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "alpha channel"));
		vbox.add(p);
		saveAlphaCB = new JCheckBox("save alpha");
		p.add(saveAlphaCB);
		saveAlphaCB.setSelected(saveAlpha);
		saveAlphaCB.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				saveAlpha = ((JCheckBox) e.getSource()).isSelected();
			}
			
		});

		saveScreenshotCB = new JCheckBox("save Screenshot");
		p.add(saveScreenshotCB);
		saveScreenshotCB.setSelected(saveScreenshot);
		saveScreenshotCB.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				saveScreenshot = ((JCheckBox) e.getSource()).isSelected();
			}
			
		});

		hbox = Box.createHorizontalBox();
		accessory.add(hbox);
		hbox.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Record interval"));
		int strutwidth = 10;
		startTTF = new JTextField(Double.toString(startTime));
		startTTF.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				startTime = Double.parseDouble(startTTF.getText());
				System.err.println("Start time = "+startTime);
			}
			
		});
		hbox.add(startTTF);
		hbox.add(Box.createHorizontalStrut(strutwidth));

		endTTF = new JTextField(Double.toString(endTime));
		endTTF.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				endTime = Double.parseDouble(endTTF.getText());
				System.err.println("End time = "+endTime);
			}
			
		});
		hbox.add(endTTF);

		hbox = Box.createHorizontalBox();
		accessory.add(hbox);
		hbox.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Save directory"));
		saveDirectoryTF = new JTextField(currentDirectoryPath);
		saveDirectoryTF.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				setCurrentDirectoryPath(saveDirectoryTF.getText());
			}
			
		});
		saveDirectoryTF.setToolTipText("<html><body>Set path to directory <br>" +
		"for saving images in recording mode</body></html>");
		hbox.add(saveDirectoryTF);
		hbox.add(Box.createHorizontalStrut(strutwidth));
		JButton browse = new JButton("Browse...");
		hbox.add(browse);
		browse.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				dirchooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				int result = dirchooser.showOpenDialog(panel);
				if (result == JFileChooser.APPROVE_OPTION)	{
					File file = dirchooser.getSelectedFile();
					currentDirectoryPath = file.getAbsolutePath();
					saveDirectoryTF.setText(currentDirectoryPath);
					System.err.println("Save path = "+currentDirectoryPath);
				}
			}
			
		});
		hbox.add(Box.createHorizontalStrut(strutwidth));
		JLabel label = new JLabel("Stem:");
		hbox.add(label);
		hbox.add(Box.createHorizontalStrut(strutwidth));
		stemTF = new JTextField(stemName);
		stemTF.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				stemName = stemTF.getText();
			}
			
		});
		stemTF.setToolTipText("<html><body>Choose the stem name<br>" +
		"for saving images in recording mode</body></html>");
		hbox.add(stemTF);
		label = new JLabel("Start count:");
		hbox.add(label);
		hbox.add(Box.createHorizontalStrut(strutwidth));
		final JTextField scF = new JTextField(Integer.toString(startCount));
		scF.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				startCount = Integer.parseInt(scF.getText());
			}
			
		});
		scF.setToolTipText("<html><body>Choose the start count<br>" +
		"for numbering images in recording mode</body></html>");
		hbox.add(scF);
		
		return accessory;
	}
	
	private Object readResolve()	{
		System.err.println("RecPrefs: in readResolve()");
		System.err.println("aa = "+antialiasing);
		System.err.println("dim = "+dimension);
		setCurrentDirectoryPath(currentDirectoryPath);
		init();
		return this;
	}

	public String getFileFormatSuffix() {
		return fileFormatSuffix;
	}

	public void setFileFormatSuffix(String fileFormatSuffix) {
		this.fileFormatSuffix = fileFormatSuffix;
	}
	
//	public void renderOffscreen(AbstractViewer viewer, int tick)	{
//		String num = String.format(frameCountFormatString,tick);
//		File outfile = new File(getCurrentDirectoryPath()+"/"+getStemName()+num+"."+
//				getFileFormatSuffix());
//		System.err.println("outfile = "+outfile.getAbsolutePath());
//		System.err.println("dim = "+dimension+" aa: "+antialiasing);
//		if ( !(viewer instanceof de.jreality.jogl.AbstractViewer)) 
//			throw new IllegalArgumentException("Can only render offscreen with jogl backend");
//		img = viewer.renderOffscreen(antialiasing*dimension.width, antialiasing*dimension.height);
//		if (img2 == null || img2.getWidth() != dimension.width || img2.getHeight() != img2.getHeight())
//			img2 = new BufferedImage(
//				dimension.width, 
//				dimension.height,
//				isSaveAlpha() ? BufferedImage.TYPE_INT_ARGB :  BufferedImage.TYPE_INT_RGB);
//		Graphics2D g = (Graphics2D) img2.getGraphics();
//		g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
//		img2.getGraphics().drawImage(
//				img.getScaledInstance(
//						dimension.width,
//						dimension.height,
//						BufferedImage.SCALE_SMOOTH
//				),
//				0,
//				0,
//				null
//		);
//		img.coerceData(true);
//		ImageUtility.writeBufferedImage(outfile, img2);
//
//	}

	public String getFrameCountFormatString() {
		return frameCountFormatString;
	}

	public void setFrameCountFormatString(String frameCountFormatString) {
		this.frameCountFormatString = frameCountFormatString;
	}

	public boolean isSaveScreenshot() {
		return saveScreenshot;
	}

	public int getStartCount() {
		return startCount;
	}

	public void setStartCount(int startCount) {
		this.startCount = startCount;
	}


}