package charlesgunn.anim.jreality;

import java.util.List;
import java.util.Vector;

import charlesgunn.anim.core.KeyFrameAnimated;
import charlesgunn.anim.core.KeyFrameAnimatedBean;
import charlesgunn.anim.core.KeyFrameAnimatedTransformation;
import charlesgunn.anim.core.TimeDescriptor;
import charlesgunn.anim.io.ImportExport;
import charlesgunn.anim.util.AnimationUtility.InterpolationTypes;
import de.jreality.math.Pn;
import de.jreality.scene.Camera;
import de.jreality.scene.SceneGraphComponent;
import de.jreality.scene.SceneGraphVisitor;
import de.jreality.scene.Transformation;
import de.jreality.scene.Viewer;
import de.jreality.shader.CommonAttributes;
import de.jreality.util.CameraUtility;
import de.jreality.util.SceneGraphUtility;
/**
 * TODO;
 * animate booleans (particularly visibility)
 * @author gunn
 *
 */
public class SceneGraphAnimator implements KeyFrameAnimated {

	public static String ANIMATED = "animated",
			LOCAL_ANIMATED = "localAnimated";  // doesn't effect recursive traversal of scene graph
	static int count = 0;
	String name = "sceneGraphAnimator";
	List<KeyFrameAnimated> animated = new Vector<KeyFrameAnimated>();
	boolean animateCamera = false;
	boolean createTransforms = false;
	boolean fromViewer = false;
	InterpolationTypes defaultInterp = InterpolationTypes.CUBIC_HERMITE;
	transient SceneGraphAnimatorVisitor sga = new SceneGraphAnimatorVisitor();
	transient SceneGraphComponent root = null;
	transient Viewer viewer = null;
	boolean writable = true,
			givesWay = true;
	
	public SceneGraphAnimator(Viewer v)	{
		viewer = v;
		root = viewer.getSceneRoot();
		fromViewer = true;
		System.err.println("Camera is "+viewer.getCameraPath());
	}

	public SceneGraphAnimator(SceneGraphComponent sgc)	{
		root = sgc;
	}
	
	public void init()	{
		if (fromViewer && animateCamera)	{
			Camera c = CameraUtility.getCamera(viewer);
			KeyFrameAnimatedBean<Camera> ab = new KeyFrameAnimatedBean<Camera>(c);
			ab.setName("cameraBean");
			animated.add(ab);					
		}
//		System.err.println("\nCalling SGC visit\n");
		sga.visit(root);
	}
	public void addKeyFrame(TimeDescriptor td) {
		for (KeyFrameAnimated a: animated)	a.addKeyFrame(td);
	}

	public void deleteKeyFrame(TimeDescriptor time) {
		for (KeyFrameAnimated a: animated)	a.deleteKeyFrame(time);
	}

	public void setCurrentValue(Object value) {
		for (KeyFrameAnimated a: animated)	a.setCurrentValue(value);
	}

	public void setValueAtTime(final double t) {
//		Scene.executeWriter(root, new Runnable() {
//
//			public void run() {
				for (KeyFrameAnimated a: animated)	a.setValueAtTime(t);
//			}
//			
//		});

	}
	
	/**
	 * TODO: fix this to remain updated to the scene graph
	 * @author Charles Gunn
	 *
	 */
	class SceneGraphAnimatorVisitor extends SceneGraphVisitor {
		int currentMetric = Pn.EUCLIDEAN;
		boolean localAnimated = true;
		Vector<SceneGraphComponent> done = new Vector<SceneGraphComponent>();
		@Override
		public void visit(SceneGraphComponent c) {
			if (done.contains(c)) return;
			System.err.println("Visiting "+c.getName());
			de.jreality.scene.Appearance ap = c.getAppearance();
			if (ap != null) {
				Object foo = ap.getAttribute(ANIMATED,Boolean.class);
				if (foo != null && foo instanceof Boolean)	{
					Boolean animated = ((Boolean) foo);
					if (!animated) 
						return;
				}
				foo = ap.getAttribute(LOCAL_ANIMATED,Boolean.class);
				if (foo != null && foo instanceof Boolean)	{
					localAnimated = ((Boolean) foo);
				} else 
					localAnimated = true;
				foo = ap.getAttribute(CommonAttributes.METRIC, Integer.class);
				if (foo != null && foo instanceof Integer)	{
					currentMetric = (Integer) foo;
//					System.err.println(c.getName()+" Sets metric = "+currentMetric);
				}
			}
			if (createTransforms && c.getTransformation() == null)	{
				c.setTransformation(new Transformation());
			}
			if (c.getTransformation() != null) {
//				System.err.println(c.getName()+" ASG: metric = "+currentMetric);
				Transformation t = c.getTransformation();
				if (!t.isReadOnly() && localAnimated) {
					KeyFrameAnimatedTransformation T = new KeyFrameAnimatedTransformation(t, currentMetric);	
					T.setInterpolationType(defaultInterp);
					T.setName(c.getName()+" Tform");
					System.err.println("Adding keyframe for "+T.getName());
					animated.add(T);					
				}
			}
			done.add(c);
			c.childrenAccept(this);
		}

//		@Override
//		public void visit(Transformation t) {
//			System.err.println("Animating tform "+t.getName());
//		}
		
	}

	public void setFromArchive(KeyFrameAnimated archived) {
		if ( !(archived instanceof SceneGraphAnimator)) {
			throw new IllegalArgumentException("wrong class");
		}
		SceneGraphAnimator archSGA = (SceneGraphAnimator) archived;
		animateCamera = archSGA.animateCamera;
		createTransforms = archSGA.createTransforms;
		defaultInterp = archSGA.defaultInterp;
		fromViewer = archSGA.fromViewer;
		List<KeyFrameAnimated> readAnimated = archSGA.animated;
		for (KeyFrameAnimated kfa : readAnimated)	{
			KeyFrameAnimated origKfa = ImportExport.elementNamed(kfa.getName(), animated);
			if (origKfa == null)	{
				System.err.println("SGA: unknown animated "+kfa.getName());
				continue;
			}
			origKfa.setFromArchive(kfa);
		}
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public boolean isWritable() {
		return writable;
	}
	public void setWritable(boolean b) {
		writable = b;
	}

	public InterpolationTypes getDefaultInterp() {
		return defaultInterp;
	}

	public void setDefaultInterp(InterpolationTypes defaultInterp) {
		this.defaultInterp = defaultInterp;
		System.err.println("SGA: Setting default interpolation to "+defaultInterp);
	}

	public boolean isAnimateCamera() {
		return animateCamera;
	}

	public void setAnimateCamera(boolean animateCamera) {
		this.animateCamera = animateCamera;
	}

	public boolean isCreateTransforms() {
		return createTransforms;
	}

	public void setCreateTransforms(boolean createTransforms) {
		this.createTransforms = createTransforms;
	}

	public void printState() {
		for (KeyFrameAnimated kfa: animated)
			kfa.printState();
	}

	public void endAnimation() {
		// TODO Auto-generated method stub
		
	}

	public void startAnimation() {
		// TODO Auto-generated method stub
		
	}

	public boolean isEqual(Object v1, Object v2) {
		return false;
	}

	public boolean isSticky() {
		return false;
	}

	public void setSticky(boolean b) {
		
	}

	@Override
	public boolean isGivesWay() {
		return givesWay;
	}

	@Override
	public void setGivesWay(boolean b) {
		givesWay = b;
		for (KeyFrameAnimated a: animated)	a.setGivesWay(givesWay);
	}

}
