package charlesgunn.anim.plugin;

import java.awt.GridLayout;
import java.util.List;
import java.util.Vector;

import charlesgunn.anim.core.Animated;
import charlesgunn.anim.core.KeyFrameAnimated;
import charlesgunn.anim.core.KeyFrameAnimatedBean;
import charlesgunn.anim.gui.AnimationPanel;
import charlesgunn.anim.gui.AnimationPanelListenerImpl;
import charlesgunn.anim.gui.AnimationPanelRecordListener;
import charlesgunn.anim.jreality.SceneGraphAnimator;
import charlesgunn.anim.util.AnimationUtility.InterpolationTypes;
import de.jreality.plugin.JRViewerUtility;
import de.jreality.plugin.basic.Content;
import de.jreality.plugin.basic.View;
import de.jreality.plugin.basic.Content.ContentChangedEvent;
import de.jreality.plugin.basic.Content.ContentChangedListener;
import de.jreality.scene.Camera;
import de.jreality.scene.Viewer;
import de.jreality.ui.viewerapp.ViewerSwitch;
import de.jreality.util.CameraUtility;
import de.jtem.jrworkspace.plugin.Controller;
import de.jtem.jrworkspace.plugin.PluginInfo;
import de.jtem.jrworkspace.plugin.sidecontainer.SideContainerPerspective;
import de.jtem.jrworkspace.plugin.sidecontainer.template.ShrinkPanelPlugin;

public class AnimationPlugin extends ShrinkPanelPlugin {
	@Override
	public String getHelpDocument() {
		// TODO Auto-generated method stub
		return super.getHelpDocument();
	}

	@Override
	public Class<?> getHelpHandle() {
		// TODO Auto-generated method stub
		return super.getHelpHandle();
	}

	@Override
	public String getHelpPath() {
		// TODO Auto-generated method stub
		return super.getHelpPath();
	}

	@Override
	public String getHelpTitle() {
		// TODO Auto-generated method stub
		return super.getHelpTitle();
	}

	AnimationPanel ap;
	Content content;
	Controller controller;
	private SceneGraphAnimator sga;
	private KeyFrameAnimatedBean<Camera> ac;
	private AnimationPanelListenerImpl animPanelListener;
	private List<Animated> animated;
	private Viewer viewer;
	transient  boolean animateSceneGraph = true, animateCamera = false;
	InterpolationTypes defaultInterp = InterpolationTypes.CUBIC_HERMITE;

	@Override
	public Class<? extends SideContainerPerspective> getPerspectivePluginClass() {
		return View.class;
	}

	public AnimationPlugin() {
		setInitialPosition(SHRINKER_TOP);
		ap = new AnimationPanel();
		animated = new Vector<Animated>();
		shrinkPanel.add(ap.getPanel());
		shrinkPanel.setLayout(new GridLayout());
	}

	public List<Animated> getAnimated() {
		return animated;
	}

	@Override
	public PluginInfo getPluginInfo() {
		PluginInfo info = new PluginInfo();
		info.name = "Animation";
		info.vendorName = "Charles Gunn";
		return info;
	}

	@Override
	public void install(Controller c) throws Exception {
		super.install(c);
		View view = c.getPlugin(View.class);
		if (viewer == null) viewer = view.getViewer();
		if (viewer instanceof ViewerSwitch) viewer = ((ViewerSwitch) viewer).getCurrentViewer();
		init();
//		content = c.getPlugin(Content.class);
		controller = c;
		content = JRViewerUtility.getContentPlugin(c);
		content.addContentChangedListener(new ContentChangedListener() {

			public void contentChanged(ContentChangedEvent cce) {
				System.err.println("AnimationPlugin: Content changed");
				if (animateSceneGraph) resetSceneGraph();
			}
			
		});
	}

	@Override
	public void uninstall(Controller c) throws Exception {
		// TODO Auto-generated method stub
		super.uninstall(c);
	}

	public AnimationPanel getAnimationPanel() {
		return ap;
	}

	public SceneGraphAnimator getSga() {
		return sga;
	}

	public  void init()	{
		
		animPanelListener = new AnimationPanelListenerImpl(viewer, "application");
//		animPanelListener.setViewer(viewer);
		animPanelListener.setAnimated(animated);
		ap.addAnimationPanelListener(animPanelListener);
		AnimationPanelRecordListener aprl = new AnimationPanelRecordListener(viewer, "record");
		ap.addAnimationPanelListener(aprl);

	}

	public void update() {
		resetSceneGraph();
		animateCamera();
	}
	private void animateCamera() {
		if (ac != null && animated.contains(ac)) animated.remove(ac);
		if (!animateCamera) return;
		Camera c = CameraUtility.getCamera(viewer);
		ac = new KeyFrameAnimatedBean<Camera>(c);
		ac.setName("cameraBean");
		animated.add(ac);
	}

	public void resetSceneGraph() {
		if (sga != null && animated.contains(sga)) animated.remove(sga);
		if (!animateSceneGraph) return;
		
		sga = new SceneGraphAnimator(viewer.getSceneRoot());
		sga.setDefaultInterp(defaultInterp);
		// we animate the camera here
//		sga.setAnimateCamera(animateCamera); //animateCamera);
		sga.setName("sga");
		sga.init();
		
		animated.add(sga);
	}

	public boolean isAnimateSceneGraph() {
		return animateSceneGraph;
	}

	public void setAnimateSceneGraph(boolean animateSceneGraph) {
		this.animateSceneGraph = animateSceneGraph;
		// this is commented out since it leads to freeze-up for certain scenes
		resetSceneGraph();
	}

	public boolean isAnimateCamera() {
		return animateCamera;
	}

	public void setAnimateCamera(boolean animateCamera) {
		this.animateCamera = animateCamera;
		animateCamera();
	}

	public InterpolationTypes getDefaultInterp() {
		return defaultInterp;
	}

	public void setDefaultInterp(InterpolationTypes defaultInterp) {
		this.defaultInterp = defaultInterp;
		if (sga != null) sga.setDefaultInterp(defaultInterp);
	}

	public Viewer getViewer() {
		return viewer;
	}

	public void setViewer(Viewer viewer) {
		this.viewer = viewer;
	}


}
