package charlesgunn.anim.sets;

import charlesgunn.anim.util.AnimationUtility;

public class AbstractAnimatedSet {

	protected double[][] dkeySet;
	protected double[][] keySet;
	protected AnimationUtility.InterpolationTypes[] interpType;
	protected AnimationUtility.BoundaryModes[] wrapType;

	public AbstractAnimatedSet(double[][] dkeySet, int[] interpType)	{
		this(dkeySet, convertOldToNew(interpType));
	}
	public AbstractAnimatedSet(double[][] dkeySet, AnimationUtility.InterpolationTypes[] interpType)	{
		super();
		this.dkeySet = dkeySet;
		if (interpType == null)	{
			this.interpType = new AnimationUtility.InterpolationTypes[dkeySet.length];
			for (int i = 0; i<this.interpType.length; ++i) 
				this.interpType[i] = AnimationUtility.InterpolationTypes.CUBIC_HERMITE;
		} else
			this.interpType = interpType;
		wrapType = new AnimationUtility.BoundaryModes[dkeySet.length];
		for (int i = 0; i<wrapType.length; ++i) 
			wrapType[i] = AnimationUtility.BoundaryModes.CLAMP;

		initializeKeys();

	}
	
	private static AnimationUtility.InterpolationTypes[] convertOldToNew(int[] interpType)	{
		AnimationUtility.InterpolationTypes[] newtype = new AnimationUtility.InterpolationTypes[interpType.length];
		for (int i = 0; i<interpType.length; ++i)	{
			switch(interpType[i])	{
			case 0:
			default:
				newtype[i] = AnimationUtility.InterpolationTypes.CONSTANT;
			case 1:
				newtype[i] = AnimationUtility.InterpolationTypes.LINEAR;
			case 2:
				newtype[i] = AnimationUtility.InterpolationTypes.CUBIC_HERMITE;
			}
		}
		return newtype;
	}
	protected void initializeKeys() {
		keySet = new double[dkeySet.length][];
		for (int j = 0; j<dkeySet.length; ++j)	{
			keySet[j] = new double[dkeySet[j].length];
			keySet[j][0] = dkeySet[j][0];
			for (int i = 1; i<dkeySet[j].length; ++i)	{
				keySet[j][i] = keySet[j][i-1] +dkeySet[j][i];
			}
		}
		
	}

	public double getTMin() {
		return keySet[0][0];
	}

	public double getTMax() {
		return keySet[0][keySet[0].length-1];
	}
	
	public void setWrapType(AnimationUtility.BoundaryModes[] wrap)	{
		wrapType = wrap;
	}
	
	public void setWrapType(AnimationUtility.BoundaryModes wrap)	{
		for (int i =0; i<wrapType.length; ++i) wrapType[i] = wrap;
	}
	
	public static double[][] transpose(double[][] array)	{
		double[][] transp = new double[array[0].length][array.length];
		for (int i = 0; i<array.length; ++i)	{
			for (int j = 0; j<array.length; ++j)	{
				transp[j][i] = array[i][j];
			}
		}
		return array;
	}

}
