package charlesgunn.anim.sets;

import charlesgunn.anim.util.AnimationUtility;
import charlesgunn.anim.util.AnimationUtility.InterpolationTypes;
import de.jreality.math.Rn;

public class AnimatedDoubleArrayArraySet {
	
	public double[] getKeySet() {
		return keySet;
	}

	double[] dkeySet, keySet;
	double[][][] valueSet;
	AnimationUtility.InterpolationTypes interpType;
	public AnimatedDoubleArrayArraySet(double[] dkeySet, double[][][] valueSet)	{
		this(dkeySet, valueSet, InterpolationTypes.LINEAR);
	}
	public AnimatedDoubleArrayArraySet(double[] dkeySet, double[][][] valueSet, AnimationUtility.InterpolationTypes interpType)	{
		this.dkeySet = dkeySet;
		this.valueSet = valueSet;
		this.interpType = interpType;
		initializeKeys();
	}
	
	void initializeKeys()	{
		keySet = new double[dkeySet.length];
		keySet[0] = dkeySet[0];
		for (int i = 1; i<dkeySet.length; ++i)	{
			keySet[i] = keySet[i-1] +dkeySet[i];
		}
		System.err.println("keys = "+Rn.toString(keySet));
	}
	
	public double getTMin()	{
		return keySet[0];
	}
	
	public double getTMax()	{
		return keySet[keySet.length-1];
	}
	
	public double[][] getValuesAtTime(double t, double[][] vals)	{
//		if (vals == null ) vals = new double[valueSet[0].length][];
		int n = keySet.length;
		int key = -1;
		
		int i = 0;
		// find the pair of keys (key, key+1) such that t is contained 
		// in the half-open interval [key, key+1)
		while ( i < n && t >= keySet[i] ) i++;
		if (i == 0) return valueSet[0];
		else if (i >= n) return valueSet[n-1];
		else key = i-1;
		System.err.println("Time = "+t+"key = "+key);
		switch(interpType)	{
			case CONSTANT:
			default:
				vals = valueSet[key];
				break;
			case LINEAR:
				vals = AnimationUtility.linearInterpolation(null, t, keySet[key], keySet[key+1], valueSet[key], valueSet[key+1]);
				break;
			case CUBIC_HERMITE:
				vals = AnimationUtility.hermiteInterpolation(t, keySet[key], keySet[key+1], valueSet[key], valueSet[key+1]);
				break;
		}
		return vals;
	}
	public double[][][] getValueSet() {
		return valueSet;
	}
}
