package charlesgunn.anim.sets;

import charlesgunn.anim.util.AnimationUtility;

public class AnimatedDoubleArraySet {
	
	double[] dkeySet, keySet;
	double[][] valueSet;
	AnimationUtility.InterpolationTypes interpType;
	public AnimatedDoubleArraySet(double[] dkeySet, double[][] valueSet)	{
		this(dkeySet, valueSet, null);
	}
	public AnimatedDoubleArraySet(double[] dkeySet, double[][] valueSet, AnimationUtility.InterpolationTypes interpType)	{
		this.dkeySet = dkeySet;
		this.valueSet = valueSet;
		this.interpType = interpType;
		initializeKeys();
	}
	
	void initializeKeys()	{
		keySet = new double[dkeySet.length];
		keySet[0] = dkeySet[0];
		for (int i = 1; i<dkeySet.length; ++i)	{
				keySet[i] = keySet[i-1] +dkeySet[i];
		}
		
	}
	
	public double getTMin()	{
		return keySet[0];
	}
	
	public double getTMax()	{
		return keySet[keySet.length-1];
	}
	
	public double[] getValuesAtTime(double t, double[] vals)	{
		if (vals == null || vals.length != valueSet.length) vals = new double[valueSet.length];
//		for (int j = 0; j<keySet.length; ++j)	{
			int key = -1;
			int n = keySet.length;
			t = keySet[0] + (( (t-keySet[0]) % (keySet[n-1]-keySet[0])));
			for (int i = 1; i<n; ++i)	{
				if (t < keySet[i]) { key = i-1; break;}
			} 			
			if (key == -1)  {
				vals = valueSet[n-1];
			}
			switch(interpType)	{
			case CONSTANT:
			default:
				vals = valueSet[key];
				break;
			case LINEAR:
				vals = AnimationUtility.linearInterpolation(t, keySet[key], keySet[key+1], valueSet[key], valueSet[key+1]);
				break;
			case CUBIC_HERMITE:
				vals = AnimationUtility.hermiteInterpolation(t, keySet[key], keySet[key+1], valueSet[key], valueSet[key+1]);
				break;
			}
		return vals;
	}
}
