package charlesgunn.anim.sets;

import charlesgunn.anim.util.AnimationUtility;
import charlesgunn.anim.util.AnimationUtility.InterpolationTypes;

public class AnimatedDoubleSet extends AbstractAnimatedSet {
	double[][] valueSet;
	InterpolationTypes type = InterpolationTypes.CONSTANT;
	public AnimatedDoubleSet(double[][] dkeySet, double[][] valueSet)	{
		this(dkeySet, valueSet, (AnimationUtility.InterpolationTypes[]) null);
	}
	public AnimatedDoubleSet(double[][] dkeySet, double[][] valueSet, AnimationUtility.InterpolationTypes[] interpType)	{
		super(dkeySet, interpType);
		this.valueSet = valueSet;
	}
	
	public AnimatedDoubleSet(double[] dkeySet, double[] valueSet, AnimationUtility.InterpolationTypes interpType)	{
		this(new double[][]{dkeySet}, new double[][]{valueSet}, new InterpolationTypes[]{interpType});
	}
	
	public AnimatedDoubleSet(double[][] dkeySet, double[][] valueSet, int[] interpType)	{
		super(dkeySet, interpType);
		this.valueSet = valueSet;
	}
	
	public void setInterpolationType(InterpolationTypes type)	{
		this.type = type;
	}
	public double[] getValuesAtTime(double t, double[] vals)	{
		if (vals == null || vals.length != valueSet.length) vals = new double[valueSet.length];
		for (int j = 0; j<keySet.length; ++j)	{
			int key = -1;
			int n = keySet[j].length;
			for (int i = 1; i<n; ++i)	{
				if (t < keySet[j][i]) { key = i-1; break;}
			} 			
			if (key == -1)  {
				vals[j] = valueSet[j][n-1];
				continue;
			}
			switch(interpType[j])	{
			case CONSTANT:
			default:
				vals[j] = valueSet[j][key];
				break;
			case LINEAR:
				vals[j] = AnimationUtility.linearInterpolation(t, keySet[j][key], keySet[j][key+1], valueSet[j][key], valueSet[j][key+1]);
				break;
			case CUBIC_HERMITE:
				vals[j] = AnimationUtility.hermiteInterpolation(t, keySet[j][key], keySet[j][key+1], valueSet[j][key], valueSet[j][key+1]);
				break;
			}
		}
		return vals;
	}
}
