package de.jtem.discretegroup.groups;

import de.jreality.math.P3;
import de.jreality.math.Pn;
import de.jreality.math.Rn;
import de.jtem.discretegroup.core.DiscreteGroup;
import de.jtem.discretegroup.core.DiscreteGroupElement;
import de.jtem.discretegroup.core.DiscreteGroupUtility;

public class Spherical3DGroup {

	String[] names = {"343","335","533"};		// 24-, 120-, and 600-cell
	
	public static DiscreteGroup instanceOf(String name)	{
		if (name.equals("335")) return the600Cell();
		else if (name.equals("533")) return the120Cell();
		else if (name.equals("433")) return the24Cell();
		else throw new IllegalArgumentException("Not yet implemented: "+name);
	}

	public static DiscreteGroup the600Cell() {
		double[][] cc = { {1, -1, 1, 4/.944272},
				{-1, 1, 1, 4/.944272},
				{1, 1, -1, 4/.944272},
				{-1, -1, -1, 4/.944272}};
		DiscreteGroupElement[] gens = new DiscreteGroupElement[14];
		int count = 0;
		for (int i = 0; i<4; ++i)	{
			for (int j = i; j<4; ++j)	{
				if (i == j ) continue;
				double[] screw = P3.makeScrewMotionMatrix(null, cc[i], cc[j], Math.PI/5, Pn.ELLIPTIC);
				gens[count] = new DiscreteGroupElement(Pn.ELLIPTIC, screw);
				gens[count].setWord(DiscreteGroupUtility.genNames[count]);
				gens[count+6] = gens[count].getInverse();
				count++;
			}
		}
		double[] screw = P3.makeRotationMatrix(null, cc[0], cc[1], 2*Math.PI/5, Pn.ELLIPTIC);
		gens[12] = new DiscreteGroupElement(Pn.ELLIPTIC, screw);
		gens[12].setWord(DiscreteGroupUtility.genNames[count]);
		gens[13] = gens[12].getInverse();
		DiscreteGroup dg = new DiscreteGroup();
		dg.setDimension(3);
		dg.setMetric(Pn.ELLIPTIC);
		dg.setGenerators(gens);
		dg.setFinite(true);
		
		dg.update();
		System.err.println("# elements = "+dg.getElementList().length);
		return dg;
		
	}

	public static DiscreteGroup the120Cell() {
		double[][] cc = { {1, -1, 1, 4/.944272},
				{-1, 1, 1, 4/.944272},
				{1, 1, -1, 4/.944272},
				{-1, -1, -1, 4/.944272}};
		DiscreteGroupElement[] gens = new DiscreteGroupElement[12];
		int count = 0;
		for (int i = 0; i<4; ++i)	{
			for (int j = i; j<4; ++j)	{
				if (i == j ) continue;
				double[] screw = P3.makeScrewMotionMatrix(null, cc[i], cc[j], Math.PI/5, Pn.ELLIPTIC);
				gens[count] = new DiscreteGroupElement(Pn.ELLIPTIC, screw);
				gens[count].setWord(DiscreteGroupUtility.genNames[count]);
				gens[count+6] = gens[count].getInverse();
				count++;
			}
		}
		DiscreteGroup dg = new DiscreteGroup();
		dg.setDimension(3);
		dg.setMetric(Pn.ELLIPTIC);
		dg.setGenerators(gens);
		dg.setFinite(true);
		
		dg.update();
		System.err.println("# elements = "+dg.getElementList().length);
		return dg;
		
	}

	private static DiscreteGroup the24Cell() {
		double[][] cc = {{0,0,0,1},
				{Math.PI/2, 0, 0, 1},
				{0, Math.PI/2, 0, 1},
				{0,0,Math.PI/2,1}};
		DiscreteGroupElement[] gens = new DiscreteGroupElement[6];
		int count = 0;
		double[][] matrices = {
				{0,0,0,1,
				 0,0,1,0,
				 0,-1,0,0,
				 -1,0,0,0
				},
				{0,0,0,1,
					 0,0,0,1,
					 -1,0,0,0,
					 0,-1,0,0
					},
				{0,1,0,0,
				-1,0,0,0,
				0,0,0,1,
				0,0,-1,0
				}
				
		};
		for (int i = 0; i<3; ++i)	{
				double[] screw = matrices[i]; //P3.makeScrewMotionMatrix(null, cc[0], cc[i+1], Math.PI/2, Pn.ELLIPTIC);
				System.err.println("matrix = "+Rn.matrixToString(screw));
				gens[count] = new DiscreteGroupElement(Pn.ELLIPTIC, screw);
				gens[count].setWord(DiscreteGroupUtility.genNames[count]);
				gens[count+3] = gens[count].getInverse();
				count++;
		}
		DiscreteGroup dg = new DiscreteGroup();
		dg.setDimension(3);
		dg.setMetric(Pn.ELLIPTIC);
		dg.setGenerators(gens);
		dg.setFinite(true);
		
		dg.update();
		System.err.println("# elements = "+dg.getElementList().length);
		return dg;
		
	}

	public static DiscreteGroup towerOfTetrahedra()	{
		DiscreteGroup dg = new DiscreteGroup();
		dg.setDimension(3);
		dg.setMetric(Pn.ELLIPTIC);
		dg.setFinite(true);
		dg.setName("tetrahedraTower");
		DiscreteGroupElement[] ogens = the600Cell().getGenerators(),
				gens = {ogens[3]};
		dg.setGenerators(gens);
		dg.update();
		DiscreteGroupElement[] justTen = dg.getElementList(),
				allThirty = new DiscreteGroupElement[30];
		double[][] order5 = {Rn.identityMatrix(4), ogens[12].getArray(), Rn.times(null, ogens[12].getArray(), ogens[12].getArray())};
		for (int i = 0; i< justTen.length; ++i) {
			for (int j = 0; j< 3; ++j) {
				double[] m = Rn.times(null, justTen[i].getArray(), order5[j]);
				allThirty[i+(j)*10] = new DiscreteGroupElement(Pn.ELLIPTIC, m);
				allThirty[i+(j)*10].setColorIndex(j);
				allThirty[i+(j)*10].setWord(i+":"+j);
			}
		}
		dg.setElementList(allThirty);
		dg.setGenerators(null);
		return dg;
	}

	public static void main(String[] args) {
		DiscreteGroup the24Cell = the24Cell();
		
	}
}
